<?php
/**
 * Copyright (C) 2014-2017 ServMask Inc.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * ███████╗███████╗██████╗ ██╗   ██╗███╗   ███╗ █████╗ ███████╗██╗  ██╗
 * ██╔════╝██╔════╝██╔══██╗██║   ██║████╗ ████║██╔══██╗██╔════╝██║ ██╔╝
 * ███████╗█████╗  ██████╔╝██║   ██║██╔████╔██║███████║███████╗█████╔╝
 * ╚════██║██╔══╝  ██╔══██╗╚██╗ ██╔╝██║╚██╔╝██║██╔══██║╚════██║██╔═██╗
 * ███████║███████╗██║  ██║ ╚████╔╝ ██║ ╚═╝ ██║██║  ██║███████║██║  ██╗
 * ╚══════╝╚══════╝╚═╝  ╚═╝  ╚═══╝  ╚═╝     ╚═╝╚═╝  ╚═╝╚══════╝╚═╝  ╚═╝
 */

class Ai1wmoe_Export_Upload {

	public static function execute( $params, ServMaskOneDriveClient $onedrive = null, ServMaskOneDriveCurl $adapter = null ) {

		// Set completed flag
		$params['completed'] = false;

		// Set start bytes
		if ( ! isset( $params['startBytes'] ) ) {
			$params['startBytes'] = 0;
		}

		// Set total bytes
		if ( ! isset( $params['totalBytes'] ) ) {
			$params['totalBytes'] = ai1wm_archive_bytes( $params );
		}

		// Set retry
		if ( ! isset( $params['retry'] ) ) {
			$params['retry'] = 0;
		}

		// Set OneDrive client
		if ( is_null( $onedrive ) ) {
			$onedrive = new ServMaskOneDriveClient(
				get_option( 'ai1wmoe_onedrive_token' ),
				get_option( 'ai1wmoe_onedrive_ssl', true )
			);
		}

		// Get archive file
		$archive = fopen( ai1wm_archive_path( $params ), 'rb' );

		// Read file chunk
		if ( ( fseek( $archive, $params['startBytes'] ) !== -1 )
				&& ( $chunk = fread( $archive, ServMaskOneDriveClient::CHUNK_SIZE ) ) ) {

			// Set chunk size
			$params['chunkBytes'] = ftell( $archive ) - $params['startBytes'];

			try {

				// Increase number of retries
				$params['retry'] += 1;

				// Upload file chunk
				$onedrive->uploadFileChunk( $chunk, $params, $adapter );

			} catch ( Exception $e ) {
				// Retry 3 times
				if ( $params['retry'] <= 3 ) {
					return $params;
				}

				throw $e;
			}

			// Unset retry counter
			unset( $params['retry'] );

			// Next start bytes
			$params['startBytes'] += $params['chunkBytes'];

			// Set archive details
			$name  = ai1wm_archive_name( $params );
			$bytes = ai1wm_archive_bytes( $params );
			$size  = ai1wm_archive_size( $params );

			// Get progress
			if ( isset( $params['startBytes'] ) ) {
				$progress = (int) ( ( $params['startBytes'] / $bytes ) * 100 );
			} else {
				$progress = 100;
			}

			// Set progress
			Ai1wm_Status::info(
				sprintf(
					__(
						'<i class="ai1wm-icon-onedrive"></i> ' .
						'Uploading <strong>%s</strong> (%s)<br />%d%% complete',
						AI1WMOE_PLUGIN_NAME
					),
					$name,
					$size,
					$progress
				)
			);

		} else {

			// Set last backup date
			update_option( 'ai1wmoe_onedrive_timestamp', time() );

			// Set progress
			Ai1wm_Status::done(
				__( 'Your WordPress archive has been uploaded to OneDrive.', AI1WMOE_PLUGIN_NAME ),
				__( 'OneDrive', AI1WMOE_PLUGIN_NAME )
			);

			self::notify_admin_of_new_backup( $params );

			// Unset upload URL
			unset( $params['uploadUrl'] );

			// Unset start bytes
			unset( $params['startBytes'] );

			// Unset total bytes
			unset( $params['totalBytes'] );

			// Unset chunk bytes
			unset( $params['chunkBytes'] );

			// Unset completed flag
			unset( $params['completed'] );
		}

		// Close the archive file
		fclose( $archive );

		return $params;
	}

	public static function is_notification_enabled( $recipient ) {
		return (
			get_option( 'ai1wmoe_onedrive_notify_toggle', false ) &&
			! empty( $recipient ) &&
			function_exists( 'wp_mail' )
		);
	}

	public static function notify_admin_of_new_backup( $params ) {
		$recipient = get_option( 'ai1wmoe_onedrive_notify_email', get_option( 'admin_email', '' ) );
		if ( self::is_notification_enabled( $recipient ) ) {
			$subject = __( 'OneDrive Backup report', AI1WMOE_PLUGIN_NAME );
			$message = sprintf( __( "Your site %s was successfully exported to OneDrive.\n", AI1WMOE_PLUGIN_NAME ), site_url() );
			$message .= "\n\n";
			$message .= sprintf( __( "Date: %s\n", AI1WMOE_PLUGIN_NAME ), date( 'r' ) );
			$message .= sprintf( __( "Backup file: %s\n", AI1WMOE_PLUGIN_NAME ), ai1wm_archive_name( $params ) );
			$message .= sprintf( __( "Size: %s\n", AI1WMOE_PLUGIN_NAME ), ai1wm_archive_size( $params ) );
			wp_mail( $recipient, $subject, $message );
		}
	}
}
