<?php

require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'ServMaskOneDriveCurl.php';

class ServMaskOneDriveClient
{
	const API_URL       = 'https://graph.microsoft.com/v1.0';

	const API_TOKEN_URL = 'https://servmask.com/redirect/onedrive-graph';

	const CHUNK_SIZE    = 4194304; // 4 MB

	/**
	 * OAuth Refresh Token
	 *
	 * @var string
	 */
	protected $refreshToken = null;

	/**
	 * SSL Mode
	 *
	 * @var boolean
	 */
	protected $ssl = null;

	/**
	 * Chunk stream
	 *
	 * @var resource
	 */
	protected $chunkStream = null;

	public function __construct($refreshToken, $ssl = true) {
		$this->refreshToken = $refreshToken;
		$this->ssl = $ssl;
	}

	/**
	 * List root drive
	 *
	 * @return array
	 */
	public function listDrive() {
		$api = new ServMaskOneDriveCurl;
		$api->setAccessToken($this->getAccessToken());
		$api->setSSL($this->ssl);
		$api->setBaseURL(self::API_URL);
		$api->setPath('/me/drive/root/children');

		return $api->makeRequest();
	}

	/**
	 * List specific folder
	 *
	 * @param  string $path Path
	 * @return array
	 */
	public function listFolder($path) {
		$api = new ServMaskOneDriveCurl;
		$api->setAccessToken($this->getAccessToken());
		$api->setSSL($this->ssl);
		$api->setBaseURL(self::API_URL);
		$api->setPath("/me/drive/root:/{$path}:/children");

		return $api->makeRequest();
	}

	/**
	 * Create folder (in root)
	 *
	 * @param  string $name Name
	 * @return array
	 */
	public function createFolder($name) {
		$api = new ServMaskOneDriveCurl;
		$api->setAccessToken($this->getAccessToken());
		$api->setSSL($this->ssl);
		$api->setBaseURL(self::API_URL);
		$api->setPath('/me/drive/root/children');
		$api->setHeader('Content-Type', 'application/json');
		$api->setOption(CURLOPT_POST, true);
		$api->setOption(CURLOPT_POSTFIELDS, json_encode(array(
			'name'   => $name,
			'folder' => (object) array(),
		)));

		return $api->makeRequest();
	}

	/**
	 * Download file
	 *
	 * @param  resource $fileStream File stream
	 * @param  string   $pathname   File path
	 * @return array
	 */
	public function downloadFile($fileStream, $pathname) {
		$api = new ServMaskOneDriveCurl;
		$api->setAccessToken($this->getAccessToken());
		$api->setSSL($this->ssl);
		$api->setBaseURL(self::API_URL);
		$api->setOption(CURLOPT_FILE, $fileStream);
		$api->setPath("/me/drive/root:/{$pathname}:/content");

		return $api->makeRequest();
	}

	/**
	 * Download file chunk
	 *
	 * @param  string   $pathname   File path
	 * @param  resource $fileStream File stream
	 * @param  array    $params     File parameters
	 * @return array
	 */
	public function downloadFileChunk($pathname, $fileStream, &$params = array(), $adapter = null) {
		$this->chunkStream = fopen('php://temp', 'wb+');

		if (is_null($adapter)) {
			$adapter = new ServMaskOneDriveCurl;
		}

		$adapter->setAccessToken($this->getAccessToken());
		$adapter->setSSL($this->ssl);
		$adapter->setBaseURL(self::API_URL);
		$adapter->setPath("/me/drive/root:/{$pathname}:/content");
		$adapter->setOption(CURLOPT_WRITEFUNCTION, array($this, 'curlWriteFunction'));
		$adapter->setHeader('Range', "bytes={$params['startBytes']}-{$params['endBytes']}");

		// Make request
		$adapter->makeRequest();

		// Copy chunk data into file stream
		if (fwrite($fileStream, stream_get_contents($this->chunkStream, -1, 0)) === false) {
			throw new Exception('Unable to save the file from OneDrive');
		}

		// Close chunk stream
		fclose($this->chunkStream);

		// Next startBytes
		if ($params['totalBytes'] < ($params['startBytes'] + self::CHUNK_SIZE)) {
			$params['startBytes'] = $params['totalBytes'];
		} else {
			$params['startBytes'] = $params['endBytes'] + 1;
		}

		// Next endBytes
		if ($params['totalBytes'] < ($params['endBytes'] + self::CHUNK_SIZE)) {
			$params['endBytes'] = $params['totalBytes'];
		} else {
			$params['endBytes'] += self::CHUNK_SIZE;
		}

		return $params;
	}

	/**
	 * Curl write function callback
	 *
	 * @param  resource $ch   Curl handler
	 * @param  string   $data Curl data
	 * @return integer
	 */
	public function curlWriteFunction($ch, $data) {
		$code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
		if ($code !== 200 && $code !== 206) {
			throw new Exception(sprintf('Unable to connect to OneDrive. Error code: %d', $code), $code);
		}

		// Write data to stream
		fwrite($this->chunkStream, $data);

		return strlen($data);
	}

	/**
	 * Upload file
	 *
	 * @param  resource $fileStream File stream
	 * @param  integer  $inSize     File size
	 * @param  string   $pathname   Path name
	 * @return array
	 */
	public function uploadFile($fileStream, $inSize, $pathname) {
		$api = new ServMaskOneDriveCurl;
		$api->setAccessToken($this->getAccessToken());
		$api->setSSL($this->ssl);
		$api->setBaseURL(self::API_URL);
		$api->setOption(CURLOPT_PUT, true);
		$api->setOption(CURLOPT_INFILE, $fileStream);
		$api->setOption(CURLOPT_INFILESIZE, $inSize);
		$api->setPath("/me/drive/root:/{$pathname}:/content");

		return $api->makeRequest();
	}

	/**
	 * Create upload session
	 *
	 * @param  string $pathname Path name
	 * @return array
	 */
	public function uploadResumable($pathname) {
		$api = new ServMaskOneDriveCurl;
		$api->setAccessToken($this->getAccessToken());
		$api->setSSL($this->ssl);
		$api->setBaseURL(self::API_URL);
		$api->setPath("/me/drive/root:/{$pathname}:/createUploadSession");
		$api->setHeader('Content-Length', 0);
		$api->setOption(CURLOPT_POST, true);

		// Make request
		if (($response = $api->makeRequest())) {
			if (isset($response['uploadUrl']) && ($uploadUrl = $response['uploadUrl'])) {
				return $uploadUrl;
			}
		}
	}


	/**
	 * Upload file chunk
	 *
	 * @param  string $chunk  File data
	 * @param  array  $params File parameters
	 * @return array
	 */
	public function uploadFileChunk($chunk, &$params = array(), $adapter = null) {
		if (is_null($adapter)) {
			$adapter = new ServMaskOneDriveCurl;
		}

		$adapter->setSSL($this->ssl);
		$adapter->setBaseURL($params['uploadUrl']);
		$adapter->setOption(CURLOPT_CUSTOMREQUEST, 'PUT');
		$adapter->setOption(CURLOPT_POSTFIELDS, $chunk);

        // Set end range
		$endBytes = $params['startBytes'] + $params['chunkBytes'] - 1;

		// Upload chunk
		$adapter->setHeader('Content-Length', $params['chunkBytes']);
		$adapter->setHeader('Content-Range', "bytes {$params['startBytes']}-$endBytes/{$params['totalBytes']}");

		return $adapter->makeRequest();
	}

	/**
	 * Deletes a file or folder
	 *
	 * @param  string $fileId Google Drive File ID.
	 * @return mixed
	 */
	public function delete($path) {
		$api = new ServMaskOneDriveCurl;
		$api->setAccessToken($this->getAccessToken());
		$api->setSSL($this->ssl);
		$api->setBaseURL(self::API_URL);
		$api->setPath("/me/drive/root:/$path");
		$api->setOption(CURLOPT_CUSTOMREQUEST, 'DELETE');

		return $api->makeRequest();
	}

	/**
	 * Get account info
	 *
	 * @return mixed
	 */
	public function getAccountInfo() {
		$api = new ServMaskOneDriveCurl;
		$api->setAccessToken($this->getAccessToken());
		$api->setSSL($this->ssl);
		$api->setBaseURL(self::API_URL);
		$api->setPath('/me/drive');

		return $api->makeRequest();
	}

	/**
	 * Get user information
	 *
	 * @return mixed
	 */
	public function getUserInfo() {
		$api = new ServMaskOneDriveCurl;
		$api->setAccessToken($this->getAccessToken());
		$api->setSSL($this->ssl);
		$api->setBaseURL(self::API_URL);
		$api->setPath('/me');

		return $api->makeRequest();
	}

	/**
	 * Get access token
	 *
	 * @return string
	 */
	protected function getAccessToken() {
		$api = new ServMaskOneDriveCurl;
		$api->setHeader('Content-Type', 'application/json');
		$api->setSSL($this->ssl);
		$api->setBaseURL(self::API_TOKEN_URL);
		$api->setPath('/refresh');
		$api->setOption(CURLOPT_POST, true);
		$api->setOption(CURLOPT_POSTFIELDS, json_encode(array(
			'token' => $this->refreshToken,
		)));

		// Make request
		if (($data = $api->makeRequest())) {
			if (isset($data['access_token']) && ($accessToken = $data['access_token'])) {
				return $accessToken;
			}
		}

	}
}
